/*********************************************************
 * TalkClock   whmp 2014 4 06
 *
 * A Talking Clock with an invisible Button 
 * Audio  is output on pin PWM 4.
 *
 * Run the TalkClock sketch and Open the serial monitor.
 * Wait for calibration to complete.
 * Touch the plastic near the Analog Pins to hear the time.
 * Keep touching for a longer time to set the time. When you hear 'set time' stop touching, the current hour is spoken.
 * Short Touch to change the hour. Long Touch and you will hear 'set' and the current minute is spoken.
 * Short Touch to change the minute. Long Touch and you will hear 'set' . When you touch again the clock will restart.
 * 
 *
 *                      MrE@telus.net 
 **********************************************************/
#include <avr/pgmspace.h>
#include "Ambrosia.h"

// touch calibration and constants
int sensitivity=12,detect=3; 
#define LONGTOUCH 126  //max touch time = button held
#define SHORTTOUCH 20 //minimum touch time for button (debounce)  
#define ADC_START (_BV(ADEN)|_BV(ADPS1)|_BV(ADPS2)|_BV(ADPS0)|_BV(ADSC)) // enable+ set clock+ start
#define ADC_11V (_BV(REFS0)|_BV(REFS1) )// internal 1.1 v ref
#define AUDIOPORTDDR   DDRG
#define AUDIOPIN 5  // audio OC0B PWM 4
#define numberofadc 8
unsigned int index; 
int  maxval[numberofadc],wide[numberofadc],base[numberofadc], minval[numberofadc]; 

//for calculating touch data
volatile char hit,touchcount,touchtype; 
//volatile unsigned int warmup=50,setuptouch=150,touching;
volatile unsigned int warmup=500,setuptouch=5000,touching;

// for audio
volatile const prog_char * audiodata,*audiocurrent; 
volatile int audiorepeat;

//for time keeping
volatile int ticks; 
volatile char secs,mins,hours=1;

ISR(TIMER2_COMPA_vect) {
  int dif; // temp variable
  int val;
  //count ticks
  if(ticks++==15625){ // 16000000/8/128=15625 (Adjust for accuracy)
    ticks=0;
    if( secs++==60){
      secs=0;
      if( mins++==60){
        mins=0;
        if( hours++==13){
          hours=1;
        }; 
      }; 
    }; 
  }
  //play audio
  if(audiocurrent){ // sound 1/2 speed
    OCR0B=pgm_read_byte(audiocurrent++);  // set pulse width
    if(!pgm_read_byte(audiocurrent)){      // is this the end of waveform?
      if(!(--audiorepeat)){        // repeat wave form?
        audiocurrent=0;  // stop
      }
      else{
        audiocurrent=audiodata;
        if(audiorepeat==-1)    audiorepeat=0;
      }
    }
  }

  //test touch

  if((ADCSRA&_BV(ADSC))||(ticks&7)) return;//  not yet
  // read current  adc and process data
  val=ADCL; // read conversion results
  val+=ADCH<<8;
  DDRC=_BV(index); //set as output, ground and clear  charge 
  PORTC=0; 
  DDRC=0;  // and back to input
  if(!warmup){ // wait for stability
    if(setuptouch){  // update background levels
      if(val>maxval[index])maxval[index]=val;  // new max?
      if(val<minval[index])minval[index]=val;  // new min?
      base[index]=(minval[index]+maxval[index])/2;   //baseline average reading
      wide[index]=maxval[index]-minval[index];    // bandwidth
    }
    else{// touch detection
      if((dif=val-base[index])<0)dif*=-1; // absolute dif 
      if(dif>(wide[index]*10)/sensitivity)hit++; // pin triggered
    }
  }

  index=(index+1)%(numberofadc); // increment count with rollover
  ADMUX=ADC_11V|(index); //select pin
  ADCSRA=ADC_START; // start next conversion

  // after a full set of adc reads 
  if(!index) { 
    if (warmup)warmup--;
    else if(setuptouch)setuptouch--;
    else{// characterize touch
      if(hit){ //atleast 1 pin triggered
        if(hit>=detect){  // minimum for event
          touchcount++; // touch duration
          if (touchcount>LONGTOUCH){  // button is held
            touchtype=2;
            touchcount=0;
          }
        }
      }
      else{ // no triggers
        if (touchcount>SHORTTOUCH)touchtype=1; // we just released
        touchcount=0;
      }
      hit=0;
    }
  }
}
void soundsample(const prog_char * data, int repeat){
  while(audiocurrent)delay(1);
  audiodata=data;
  audiorepeat=repeat;
  audiocurrent=data;
};
void setup() { // only called once to initialize system
   int a; 
  for (a=0;a<numberofadc;a++)minval[a]=1024;// we calibrate down to detect minimums
  cli();
  AUDIOPORTDDR=_BV(AUDIOPIN); // audio output pin as output
  TCCR2A = 0; //all off
  TIMSK0=0;
  TIMSK1=0;
  TIMSK2=0;
  audiodata=0; 
  OCR2A = 127; // 16000000/8/128=15625
  TCCR2A = _BV(WGM21); //count to cmpA
  TIMSK2 = _BV(OCIE2A); //interupt at cmpA
  TCCR2B =  _BV(CS21); // clocks/8, start timmer
   // clear on match  full count 255 16000000/256=62500
 TCCR0A= _BV(WGM00) | _BV(WGM01)| _BV(COM0A1) | _BV(COM0B1) ; 
  TCCR0B=_BV(CS00);// count sys clocks, start timmer 
  sei();
  Serial.begin(9600); 
} 
void loop(){
  Serial.println("warming up");
  while(warmup);
  Serial.println("calibrating");
  while(setuptouch);
  showlevels();
  while(1){
    if(!secs)  saytime(0);
    if(secs==30){
      soundsample(beep,33);
      while(secs==30); 
    }
    checkserial();
    switch(touchtype){
    case 1:
      saytime(1);
      break;
    case 2:
      settime();
      break;
    }
  }
}
void checkserial(){
  char chin;
  if((chin=Serial.read())>0){  // check for character
    if(chin>='a'&&chin<='z') sensitivity=chin-'a'+1;  //change touch sensitivity
    if(chin>='1'&&chin<='9') detect=chin-'0';  //change detect trigger count
    if(chin==13)showlevels(); // display current values
  }
}
void printtime(){
  Serial.print("time ");
  Serial.print(hours,DEC);
  Serial.print(" ");
  Serial.print(mins,DEC);
  Serial.print(" ");
  Serial.println(secs,DEC);
}
void saytime(char mode){
  char s=secs;
  soundsample(time,1);
  snddelay(25);
  if(!hours)hours=12;
  say(hours);
  snddelay(15);
  if(mins<10){
    say(0);
    snddelay(5);
  }
  say(mins);
  if(mode){ 
    snddelay(15);
    if(s<10){
      say(0);
      snddelay(5);
    }
    say(s);
  }
  snddelay(25);
  touchtype=0;
  printtime();
}

void settime(){
  int m,h;
  m=mins;
  h=hours;
  soundsample(set,1);
  snddelay(5);
  soundsample(time,1);
  snddelay(5);
  say(h);
  touchtype=0;
  while(touchtype!=2){
    if(touchtype==1){
      h++;
      if(h==13)h=1;
      say(h);
      snddelay(5);
      touchtype=0;
    }
  }
 soundsample(set,1); 
  snddelay(50);
  say(m);
  touchtype=0;
  while(touchtype!=2){
    if(touchtype==1){
      m++;
      if(m==60)m=0;
      say (m);
      snddelay(5);
      touchtype=0;
    }
  }
soundsample(set,1); 
  snddelay(150);
  touchtype=0;
  while(!touchtype);
  secs=0;
  mins=m;
  hours=h;
  saytime(0);
}
void say(char num){ // say a number between 0 and 99 by sequencing sound samples
  switch(num){
  case  0:
    soundsample(oh,1);
    break;
  case  1:
    soundsample(one,1);
    break;
  case  2:
    soundsample(two,1); 
    break;
  case  3:
    soundsample(three,1); 
    break;
  case  4:
    soundsample(four,1); 
    break;
  case  5:
    soundsample(five,1); 
    break;
  case  6:
    soundsample(six,1); 
    break;
  case  7:
    soundsample(seven,1); 
    break;
  case  8:
    soundsample(eight,1); 
    break;
  case  9:
    soundsample(nine,1); 
    break;
  case  10:
    soundsample(ten,1); 
    break;
  case  11:
    soundsample(eleven,1); 
    break;
  case  12:
    soundsample(twelve,1); 
    break;
  case  13:
  case  15:
  case  14:
  case  16:
  case  17:
  case  18:
  case  19:
    if(num==13)   soundsample(thir,1); 
    else if(num==15)    soundsample(fif,1); 
    else say(num%10);
    snddelay(5);
    soundsample(teen,1); 
    break;
  case  20:
  case  21:
  case  22:
  case  23:
  case  24:
  case  25:
  case  26:
  case  27:
  case  28:
  case  29:
    soundsample(twen,1); 
    snddelay(5);
    soundsample(tee,1); 
    if(num==20) break;
    snddelay(10);
    say(num%10);
    break;
  case  30:
  case  31:
  case  32:
  case  33:
  case  34:
  case  35:
  case  36:
  case  37:
  case  38:
  case  39:
    soundsample(thir,1); 
    snddelay(5);
    soundsample(tee,1); 
    if(num==30) break;
    snddelay(10);
    say(num%10);
    break;
  case  40:
  case  41:
  case  42:
  case  43:
  case  44:
  case  45:
  case  46:
  case  47:
  case  48:
  case  49:
    soundsample(four,1); 
    snddelay(5);
    soundsample(tee,1); 
    if(num==40) break;
    snddelay(10);
    say(num%10);
    break;
  case  50:
  case  51:
  case  52:
  case  53:
  case  54:
  case  55:
  case  56:
  case  57:
  case  58:
  case  59:
    soundsample(fif,1); 
    snddelay(5);
    soundsample(tee,1); 
    if(num==50) break;
    snddelay(10);
    say(num%10);
    break;

  case  60:
  case  61:
  case  62:
  case  63:
  case  64:
  case  65:
  case  66:
  case  67:
  case  68:
  case  69:
    soundsample(six,1); 
    snddelay(5);
    soundsample(tee,1); 
    if(num==60) break;
    snddelay(10);
    say(num%10);
    break;
  case  70:
  case  71:
  case  72:
  case  73:
  case  74:
  case  75:
  case  76:
  case  77:
  case  78:
  case  79:
    soundsample(seven,1); 
    snddelay(5);
    soundsample(tee,1); 
    if(num==70) break;
    snddelay(10);
    say(num%10);
    break;
  case  80:
  case  81:
  case  82:
  case  83:
  case  84:
  case  85:
  case  86:
  case  87:
  case  88:
  case  89:
    soundsample(eight,1); 
    snddelay(5);
    soundsample(tee,1); 
    if(num==80) break;
    snddelay(10);
    say(num%10);
    break;
  case  90:
  case  91:
  case  92:
  case  93:
  case  94:
  case  95:
  case  96:
  case  97:
  case  98:
  case  99:
    soundsample(nine,1); 
    snddelay(5);
    soundsample(tee,1); 
    if(num==90) break;
    snddelay(10);
    say(num%10);
    break;
  }
}
void showlevels(){
  int a;
  for (a=0;a<numberofadc;a++){ 
    Serial.write(a+'0');
    Serial.print(": max "); 
    Serial.print(maxval[a]); 
    Serial.print(": min "); 
    Serial.print(minval[a]); 
    Serial.print(": base "); 
    Serial.print(base[a]); 
    Serial.print(" background "); 
    Serial.print(wide[a]/2); //normal variation from base
    Serial.print(" trigger "); 
    Serial.println((wide[a]*10)/sensitivity); // trigger level
  }
  Serial.println("a - t sensitivity 1 - 9 Detect level");
  Serial.print("Sensitivity "); 
  Serial.write(sensitivity-1+'a'); 
  Serial.print(sensitivity); 
  Serial.print(" Detect "); 
  Serial.println(detect); 
}

void snddelay(unsigned char a){ // silence is a 50% duty cycle 62.5 khz square wave !
  soundsample(silent,100*a);
}


