/*********************************************************
 * TouchBar - Arduino Proximity sensing    whmp 2014 4 06 
 * 
 * Run the TouchBar sketch and open the serial monitor 9600 return on. While   
 * TouchBar is warming up and calibrating keep your hands  away from the Arduino. 
 * After calibration is completed the calibration data ,sensitivity setting, and
 * trigger levels are shown. If you touch the header near the analog pins this
 * is detected and the profile of the touch is displayed. The sensitivity can be
 * adjusted by entering a-z, or detect level 1-9. Try r4 for high sensitivity and
 * detection at a distance. With d2 you may need to pinch the header between 
 * your fingers to detect.
 * 
 * 1122233 *  means ...
 * 
 * Pins one and three have a level 2 signal, and pin two has a level 3 signal. 
 * The * indicates the detect pin count is reached. Audio feedback is provided on
 * pin PWM 5.
 *
 *                      MrE@telus.net 
 **********************************************************/

int sensitivity=13,detect=3,setuptouch=150, warmup=100;

#define numberofadc 6  
#define ADC_START (_BV(ADEN)|_BV(ADPS1)|_BV(ADPS2)|_BV(ADPS0)|_BV(ADSC)) // enable+ set clock+ start
#define ADC_11V (_BV(REFS0)|_BV(REFS1) )// internal 1.1 v ref

// many arrays are used to make code clearer
int  analogindex,minval[numberofadc],maxval[numberofadc],val[numberofadc],wideval[numberofadc],baseval[numberofadc],diffval[numberofadc]; 

void setup() { 
  int a; 
  for (a=0;a<numberofadc;a++)minval[a]=1024;// calibrate down to detect minimums
  Serial.begin(9600); 
  Serial.println("warming up");
} 

void loop() { 
  char chin; // character in
  if((chin=Serial.read())>0){  
    if(chin>='a'&&chin<='z') sensitivity=chin-'a'+1;  //change touch sensitivity
    if(chin>='1'&&chin<='9') detect=chin-'0';  //change detect count
    if(chin==13)showlevels(); // display current values
  }
  touchbar();  // test  and report touches
  delay(8); //give a little time 
} 

void touchbar(){ // read adc's and store value,  analyze after each set of reads 
  int v;
  if(ADCSRA&_BV(ADSC))    return;// conversion not complete 
  v=ADCL; // read conversion results  low then high
  v+=ADCH<<8;
  DDRC=_BV(analogindex); //set as output to  
  PORTC=0; // ground 
  DDRC=0;  // and back to input float

  if(analogindex==8){ // time to examine data 
    // if(analogindex==numberofadc){ // change to this and you might need to adjust timing 
    testtouch();  
    analogindex=0; //next read analog 0
  }
  else{ // time to read an adc
    if(analogindex<numberofadc)  val[analogindex]=v; // store value
    analogindex++;  // next adc
  }

  ADMUX=ADC_11V|(analogindex); // read against 1.1 volt reference
  ADCSRA=ADC_START; // start next conversion
}

unsigned char testtouch(){ //  analyze an ADCset data
  unsigned char a,count=0,level=0;
  if(warmup){ // let things stabilise a bit after reset
    warmup--;
    if(!warmup) Serial.println("Calibrating"); // only happens once
    return(0);
  }
  if(setuptouch){  // find background levels
    for (a=0;a<numberofadc;a++){ 
      if(!a)  setuptouch--; // count complete adc  sets
      if(val[a]>maxval[a])maxval[a]=val[a];  // new max?
      if(val[a]<minval[a])minval[a]=val[a];  // new min?
      baseval[a]=(minval[a]+maxval[a])/2;   //baseline average reading
      wideval[a]=maxval[a]-minval[a];       // bandwidth
    }
    if(!setuptouch) showlevels(); // only happens once.
  }
  else{  // process last values and print out
    for (a=0;a<numberofadc;a++){ 
      if((diffval[a]=val[a]-baseval[a])<0)diffval[a]*=-1; // absolute dif from baseline 
      if(diffval[a]>(wideval[a]*10)/sensitivity) // sensitivity calculation
      {
        level++;
        count++;  // are we showing touch 
        Serial.write(a+'0'); 
        if(diffval[a]>(wideval[a]*15)/sensitivity) {
          level++;
          Serial.write(a+'0'); 
        }
        if(diffval[a]>(wideval[a]*20)/sensitivity) {
          level++; 
          Serial.write(a+'0');
        }
        if(diffval[a]>(wideval[a]*25)/sensitivity){
          level++; 
          Serial.write(a+'0');
        }
      }
    }
  }
  if(count){
    if(count>=detect){
      Serial.print("  *");
      tone(5,330*(level+count-detect),10);
    }
    Serial.println();
  }
  return(count); // how many pins active
}

void showlevels(){
  int a;
  for (a=0;a<numberofadc;a++){ 
    Serial.write(a+'0');
    Serial.print(": max "); 
    Serial.print(maxval[a]); 
    Serial.print(": min "); 
    Serial.print(minval[a]); 
    Serial.print(": base "); 
    Serial.print(baseval[a]); 
    Serial.print(" background "); 
    Serial.print(wideval[a]/2); //normal variation from base
    Serial.print(" trigger "); 
    Serial.println((wideval[a]*10)/sensitivity); // trigger level
  }
  Serial.println("a - t sensitivity 1 - 9 Detect level");
  Serial.print("Sensitivity "); 
  Serial.write(sensitivity-1+'a'); 
  Serial.print(sensitivity); 
  Serial.print(" Detect "); 
  Serial.println(detect); 
}



